@@ -21,7 +21,7 @@ import requests
 import sys
 
 __title__ = 'python-gitlab'
-__version__ = '0.4'
+__version__ = '0.5'
 __author__ = 'Gauvain Pocentek'
 __email__ = 'gauvain@pocentek.net'
 __license__ = 'LGPL3'
@@ -141,7 +141,7 @@ class Gitlab(object):
             raise GitlabConnectionError(
                 "Can't connect to GitLab server (%s)" % self._url)
 
-    def rawPost(self, path, data):
+    def rawPost(self, path, data=None):
         url = '%s%s' % (self._url, path)
         try:
             return requests.post(url, data,
@@ -162,6 +162,17 @@ class Gitlab(object):
             raise GitlabConnectionError(
                 "Can't connect to GitLab server (%s)" % self._url)
 
+    def rawDelete(self, path):
+        url = '%s%s' % (self._url, path)
+
+        try:
+            return requests.delete(url,
+                                   headers=self.headers,
+                                   verify=self.ssl_verify)
+        except:
+            raise GitlabConnectionError(
+                "Can't connect to GitLab server (%s)" % self._url)
+
     def list(self, obj_class, **kwargs):
         missing = []
         for k in obj_class.requiredListAttrs:
@@ -315,9 +326,9 @@ class Gitlab(object):
 
         If id is None, returns a list of hooks.
 
-        If id is an integer, test the matching hook.
+        If id is an integer, tests the matching hook.
 
-        If id is a dict, create a new object using attributes provided. The
+        If id is a dict, creates a new object using attributes provided. The
         object is NOT saved on the server. Use the save() method on the object
         to write it on the server.
         """
@@ -328,42 +339,79 @@ class Gitlab(object):
 
         If id is None, returns a list of projects.
 
-        If id is an integer, returns the matching project (or raise a
+        If id is an integer, returns the matching project (or raises a
         GitlabGetError if not found)
 
-        If id is a dict, create a new object using attributes provided. The
+        If id is a dict, creates a new object using attributes provided. The
         object is NOT saved on the server. Use the save() method on the object
         to write it on the server.
         """
         return self._getListOrObject(Project, id, **kwargs)
 
+    def UserProject(self, id=None, **kwargs):
+        """Creates a project for a user.
+
+        id must be a dict.
+        """
+        return self._getListOrObject(UserProject, id, **kwargs)
+
+    def _list_projects(self, url):
+        r = self.rawGet(url)
+        if r.status_code != 200:
+            raise GitlabListError
+
+        l = []
+        for o in r.json():
+            l.append(Project(self, o))
+
+        return l
+
+    def search_projects(self, query):
+        """Searches projects by  name.
+
+        Returns a list of matching projects.
+        """
+        return self._list_projects("/projects/search/" + query)
+
+    def all_projects(self):
+        """Lists all the projects (need admin rights)."""
+        return self._list_projects("/projects/all")
+
+    def owned_projects(self):
+        """Lists owned projects."""
+        return self._list_projects("/projects/owned")
+
     def Group(self, id=None, **kwargs):
-        """Creates/gets/lists groups(s) known by the GitLab server.
+        """Creates/gets/lists group(s) known by the GitLab server.
 
-        If id is None, returns a list of projects.
+        If id is None, returns a list of groups.
 
-        If id is an integer, returns the matching project (or raise a
+        If id is an integer, returns the matching group (or raises a
         GitlabGetError if not found)
 
-        If id is a dict, create a new object using attributes provided. The
+        If id is a dict, creates a new object using attributes provided. The
         object is NOT saved on the server. Use the save() method on the object
         to write it on the server.
         """
         return self._getListOrObject(Group, id, **kwargs)
 
     def Issue(self, id=None, **kwargs):
-        """Lists issues(s) known by the GitLab server."""
+        """Lists issues(s) known by the GitLab server.
+
+        Does not support creation or getting a single issue unlike other
+        methods in this class yet.
+        """
         return self._getListOrObject(Issue, id, **kwargs)
 
     def User(self, id=None, **kwargs):
         """Creates/gets/lists users(s) known by the GitLab server.
 
-        If id is None, returns a list of projects.
+        If id is None, returns a list of users.
 
-        If id is an integer, returns the matching project (or raise a
+        If id is an integer, returns the matching user (or raises a
         GitlabGetError if not found)
 
-        If id is a dict, create a new object using attributes provided. The
+        If id is a dict, creates a new object using attributes provided. The
         object is NOT saved on the server. Use the save() method on the object
         to write it on the server.
         """
@@ -374,7 +422,7 @@ class Gitlab(object):
 
         If id is None, returns a list of teams.
 
-        If id is an integer, returns the matching project (or raise a
+        If id is an integer, returns the matching team (or raises a
         GitlabGetError if not found)
 
         If id is a dict, create a new object using attributes provided. The
@@ -541,12 +589,27 @@ class GitlabObject(object):
         return json.dumps(self.__dict__, cls=jsonEncoder)
 
 
+class UserKey(GitlabObject):
+    _url = '/users/%(user_id)s/keys'
+    canGet = False
+    canList = False
+    canUpdate = False
+    canDelete = False
+    requiredCreateAttrs = ['user_id', 'title', 'key']
+
+
 class User(GitlabObject):
     _url = '/users'
     shortPrintAttr = 'username'
     requiredCreateAttrs = ['email', 'password', 'username', 'name']
     optionalCreateAttrs = ['skype', 'linkedin', 'twitter', 'projects_limit',
-                           'extern_uid', 'provider', 'bio']
+                           'extern_uid', 'provider', 'bio', 'admin',
+                           'can_create_group']
+
+    def Key(self, id=None, **kwargs):
+        return self._getListOrObject(UserKey, id,
+                                     user_id=self.id,
+                                     **kwargs)
 
 
 class CurrentUserKey(GitlabObject):
@@ -586,6 +649,12 @@ class Group(GitlabObject):
     requiredCreateAttrs = ['name', 'path']
     shortPrintAttr = 'name'
 
+    GUEST_ACCESS = 10
+    REPORTER_ACCESS = 20
+    DEVELOPER_ACCESS = 30
+    MASTER_ACCESS = 40
+    OWNER_ACCESS = 50
+
     def Member(self, id=None, **kwargs):
         return self._getListOrObject(GroupMember, id,
                                      group_id=self.id,
@@ -600,6 +669,7 @@ class Group(GitlabObject):
 
 class Hook(GitlabObject):
     _url = '/hooks'
+    canUpdate = False
     requiredCreateAttrs = ['url']
     shortPrintAttr = 'url'
 
@@ -679,6 +749,16 @@ class ProjectKey(GitlabObject):
     requiredCreateAttrs = ['project_id', 'title', 'key']
 
 
+class ProjectEvent(GitlabObject):
+    _url = '/projects/%(project_id)s/events'
+    canGet = False
+    canDelete = False
+    canUpdate = False
+    canCreate = False
+    requiredListAttrs = ['project_id']
+    shortPrintAttr = 'target_title'
+
+
 class ProjectHook(GitlabObject):
     _url = '/projects/%(project_id)s/hooks'
     requiredListAttrs = ['project_id']
@@ -819,6 +899,20 @@ class ProjectSnippet(GitlabObject):
                                      **kwargs)
 
 
+class UserProject(GitlabObject):
+    _url = '/projects/user/%(user_id)s'
+    _constructorTypes = {'owner': 'User', 'namespace': 'Group'}
+    canUpdate = False
+    canDelete = False
+    canList = False
+    canGet = False
+    requiredCreateAttrs = ['name', 'user_id']
+    optionalCreateAttrs = ['default_branch', 'issues_enabled', 'wall_enabled',
+                           'merge_requests_enabled', 'wiki_enabled',
+                           'snippets_enabled', 'public', 'visibility_level',
+                           'description']
+
+
 class Project(GitlabObject):
     _url = '/projects'
     _constructorTypes = {'owner': 'User', 'namespace': 'Group'}
@@ -827,7 +921,8 @@ class Project(GitlabObject):
     requiredCreateAttrs = ['name']
     optionalCreateAttrs = ['default_branch', 'issues_enabled', 'wall_enabled',
                            'merge_requests_enabled', 'wiki_enabled',
-                           'namespace_id']
+                           'snippets_enabled', 'public', 'visibility_level',
+                           'namespace_id', 'description']
     shortPrintAttr = 'path'
 
     def Branch(self, id=None, **kwargs):
@@ -840,6 +935,16 @@ class Project(GitlabObject):
                                      project_id=self.id,
                                      **kwargs)
 
+    def Event(self, id=None, **kwargs):
+        return self._getListOrObject(ProjectEvent, id,
+                                     project_id=self.id,
+                                     **kwargs)
+
+    def File(self, id=None, **kwargs):
+        return self._getListOrObject(ProjectFile, id,
+                                     project_id=self.id,
+                                     **kwargs)
+
     def Hook(self, id=None, **kwargs):
         return self._getListOrObject(ProjectHook, id,
                                      project_id=self.id,
@@ -903,6 +1008,40 @@ class Project(GitlabObject):
 
         raise GitlabGetError
 
+    def archive(self, sha=None):
+        url = '/projects/%s/repository/archive' % self.id
+        if sha:
+            url += '?sha=%s' % sha
+        r = self.gitlab.rawGet(url)
+        if r.status_code == 200:
+            return r.content
+
+        raise GitlabGetError
+
+    def create_file(self, path, branch, content, message):
+        url = "/projects/%s/repository/files" % self.id
+        url += "?file_path=%s&branch_name=%s&content=%s&commit_message=%s" % \
+            (path, branch, content, message)
+        r = self.gitlab.rawPost(url)
+        if r.status_code != 201:
+            raise GitlabCreateError
+
+    def update_file(self, path, branch, content, message):
+        url = "/projects/%s/repository/files" % self.id
+        url += "?file_path=%s&branch_name=%s&content=%s&commit_message=%s" % \
+            (path, branch, content, message)
+        r = self.gitlab.rawPut(url)
+        if r.status_code != 200:
+            raise GitlabUpdateError
+
+    def delete_file(self, path, branch, message):
+        url = "/projects/%s/repository/files" % self.id
+        url += "?file_path=%s&branch_name=%s&commit_message=%s" % \
+            (path, branch, message)
+        r = self.gitlab.rawDelete(url)
+        if r.status_code != 200:
+            raise GitlabDeleteError
+
 
 class TeamMember(GitlabObject):
     _url = '/user_teams/%(team_id)s/members'
